#include "stdafx.h"

#include "Global.h"
#include "Commands.h"

namespace SkillSystem {

  //
  // NOTE: the order of the functions here equals the internal placement order of the functions in the EXE's address space
  //

  Int32 Commands::getCurHP(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetCurHP());
    }
   
    return Int32(0);
  }

  Int32 Commands::getMaxHP(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetMaxHP());
    }
   
    return Int32(0);
  }

  Int32 Commands::getCurFatigue(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetCurFatigue());
    }
   
    return Int32(0);
  }

  Int32 Commands::getMaxFatigue(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetMaxFatigue());
    }
   
    return Int32(0);
  }

  Int32 Commands::getCurMana(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetCurMana());
    }
   
    return Int32(0);
  }

  Int32 Commands::getMaxMana(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetMaxMana());
    }
   
    return Int32(0);
  }

  Int32 Commands::getSkillTotal(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetSkillTotal());
    }
   
    return Int32(0);
  }

  Int32 Commands::getCappedSkillTotal(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetCappedSkillTotal());
    }
   
    return Int32(0);
  }

  Int32 Commands::getHPLevel(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related functions to calculate the level (percentage based)
      return Int32(int( mobile->GetCurHP() * 100 ) / mobile->GetMaxHP());
    }
   
    return Int32(0);
  }

  Int32 Commands::getFatigueLevel(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related functions to calculate the level (percentage based)
      return Int32(int( mobile->GetCurFatigue() * 100 ) / mobile->GetMaxFatigue());
    }
   
    return Int32(0);
  }

  Int32 Commands::getManaLevel(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related functions to calculate the level (percentage based)
      return Int32(int( mobile->GetCurMana() * 100 ) / mobile->GetMaxMana());
    }
   
    return Int32(0);
  }

  void Commands::setCurHP(Object ^object, Int32 NewCurHP)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      mobile->SetCurHP(int(NewCurHP), 0);
    }
  }

  void Commands::setMaxHP(Object ^object, Int32 NewMaxHP)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related functions
      mobile->SetMaxHP(int(NewMaxHP));
      mobile->HandleHealthGain();
    }
  }

  void Commands::handleHealthGain(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      mobile->HandleHealthGain();
    }
  }

  void Commands::setCurFatigue(Object ^object, Int32 NewCurFatigue)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Do not allow a negative value
      if(NewCurFatigue < 0)
        NewCurFatigue = 0;

      // Call the mobile's related function
      mobile->SetCurFatigue(int(NewCurFatigue));
    }
  }

  void Commands::setMaxFatigue(Object ^object, Int32 NewMaxFatigue)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      mobile->SetMaxFatigue(int(NewMaxFatigue));
    }
  }

  void Commands::setCurMana(Object ^object, Int32 NewCurMana)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      mobile->SetCurMana(int(NewCurMana));
    }
  }

  void Commands::setMaxMana(Object ^object, Int32 NewMaxMana)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      mobile->SetMaxMana(int(NewMaxMana));
    }
  }

  void Commands::addHP(Object ^object, Int32 HitPointsToAdd)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Do the addition
      int NewCurHP = mobile->GetCurHP() + int(HitPointsToAdd);

      // Call the mobile's related functions
      mobile->SetCurHP(NewCurHP, 0);
      mobile->HandleHealthGain();
    }
  }

  void Commands::addMana(Object ^object, Int32 ManaToAdd)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Do the addition
      int NewCurMana = mobile->GetCurMana() + int(ManaToAdd);

      // Call the mobile's related function
      mobile->SetCurMana(NewCurMana);
    }
  }

  void Commands::addFatigue(Object ^object, Int32 FatigueToAdd)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Do the addition
      int NewCurFatigue = mobile->GetCurFatigue() + int(FatigueToAdd);

      // Call the mobile's related function
      mobile->SetCurFatigue(NewCurFatigue);
    }
  }

  void Commands::loseHP(Object ^object, Int32 HitPointsToLose)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the Core's Internal Damage function to lose hit points!
      // NOTE: this means the mobile can die in the process!
      //DoDamageInternal(NULL, Mobile, HitPointsToLose, NULL, NULL);
    }
  }

  void Commands::loseMana(Object ^object, Int32 ManaToLose)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Do the substraction
      int NewCurMana = mobile->GetCurMana() - int(ManaToLose);

      // Call the mobile's related function
      mobile->SetCurMana(NewCurMana);
    }
  }

  void Commands::loseFatigue(Object ^object, Int32 FatigueToLose)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Do the substraction
      int NewCurFatigue = mobile->GetCurFatigue() - int(FatigueToLose);

      // Call the mobile's related function
      mobile->SetCurFatigue(NewCurFatigue);
    }
  }

  void Commands::restoreMobile(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Set HP, Mana and Fatigue of the mobile to their maximum values
      mobile->SetCurHP(mobile->GetMaxHP(), 0);
      mobile->SetCurMana(mobile->GetMaxMana());
      mobile->SetCurFatigue(mobile->GetMaxFatigue());
    }
  }

  void Commands::restoreFatigue(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Set fatigue of the mobile to its maximum value
      mobile->SetCurFatigue(mobile->GetMaxFatigue());
    }
  }

  void Commands::restoreMana(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Set mana of the mobile to its maximum value
      mobile->SetCurMana(mobile->GetMaxMana());
    }
  }

  void Commands::restoreHP(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Set hit points of the mobile to its maximum value
      mobile->SetCurHP(mobile->GetMaxHP(), 0);
    }
  }

  void Commands::handleWatchingSkill(Object ^object, Int32 SkillNumber)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      mobile->HandleWatchingSkill(char(SkillNumber), 100);
    }
  }

  Int32 Commands::testSkillReal(Object ^object, Int32 SkillNumber)
  {
    // NOTE: Equals skillTestReal

    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->TestSkill(char(SkillNumber), 0, 100));
    }

    return Int32(0);
  }

  Int32 Commands::skillTestReal(Object ^object, Int32 SkillNumber)
  {
    // NOTE: Equals testSkillReal

    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->TestSkill(char(SkillNumber), 0, 100));
    }

    return Int32(0);
  }

  Int32 Commands::testSkill(Object ^object, Int32 SkillNumber)
  {
    // NOTE: Equals testSkill

    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function, return 1 if the result was positive, otherwise return Int32(0)
      return Int32(mobile->TestSkill(char(SkillNumber), 0, 100) > 0 ? 1 : 0);
    }

    return Int32(0);
  }

  Int32 Commands::skillTest(Object ^object, Int32 SkillNumber)
  {
    // NOTE: Equals skillTest

    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function, return 1 if the result was positive, otherwise return Int32(0)
      return Int32(mobile->TestSkill(char(SkillNumber), 0, 100) > 0 ? 1 : 0);
    }

    return Int32(0);
  }

  Int32 Commands::getSkillSuccessChance(Object ^object, Int32 SkillNumber, Int32 Diff, Int32 Focus)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Verify Diff
      if(Diff < -1000)
        return Int32(1);
      if(Diff >  2000)
        return Int32(0);

      // Verify Focus
      if(Focus < 10 || Focus > 100)
        return Int32(0);

      // Return 0 now if the skill isn't valid/enabled
      if(!Global::Skills->IsSkillEnabled(char(SkillNumber)))
        return Int32(0);

      // After checking Diff & Focus, call the mobile's related function
      return Int32(mobile->GetSkillSuccessChance(char(SkillNumber), int(Diff), int(Focus)));
    }

    return Int32(0);
  }

  Int32 Commands::testAndlearnSkill(Object ^object, Int32 SkillNumber, Int32 Diff, Int32 Focus)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Verify Diff
      if(Diff < -1000)
        return Int32(1);
      if(Diff >  2000)
        return Int32(0);

      // Verify Focus
      if(Focus < 10 || Focus > 100)
        return Int32(0);

      // Return 0 now if the skill isn't valid/enabled
      if(!Global::Skills->IsSkillEnabled(char(SkillNumber)))
        return Int32(0);

      // After checking Diff & Focus, call the mobile's related function
      return Int32(mobile->TestAndLearnSkill(char(SkillNumber), int(Diff), int(Focus), false, 100, 1));
    }

    return Int32(0);
  }

  Int32 Commands::getRealStrength(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's relatedrelatedv function
      return Int32(int( mobile->GetRealStat(0) ));
    }

    return Int32(0);
  }

  Int32 Commands::getRealDexterity(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->GetRealStat(1) ));
    }

    return Int32(0);
  }

  Int32 Commands::getRealIntelligence(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->GetRealStat(2) ));
    }

    return Int32(0);
  }

  Int32 Commands::getStrength(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->GetStat(0) ));
    }

    return Int32(0);
  }

  Int32 Commands::getDexterity(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->GetStat(1) ));
    }

    return Int32(0);
  }

  Int32 Commands::getIntelligence(Object ^object)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->GetStat(2) ));
    }

    return Int32(0);
  }

  Int32 Commands::getStat(Object ^object, Int32 StatIndex)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->GetStat(StatIndex) ));
    }

    return Int32(0);
  }

  Int32 Commands::getRealstat(Object ^object, Int32 StatIndex)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->GetRealStat(StatIndex) ));
    }

    return Int32(0);
  }

  Int32 Commands::getStatMod(Object ^object, Int32 StatIndex)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->GetStatMod(StatIndex) ));
    }

    return Int32(0);
  }

  Int32 Commands::modifyStat(Object ^object, Int32 StatIndex, Int32 StatModifier)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      // NOTE: modifying the stat is done by modifying the stat modifier!
      return Int32(int( mobile->ModifyStatMod(StatIndex, short( StatModifier )) ));
    }

    return Int32(0);
  }

  Int32 Commands::modifyRealStat(Object ^object, Int32 StatIndex, Int32 RealStatModifier)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->ModifyRealStat(StatIndex, short( RealStatModifier )) ));
    }

    return Int32(0);
  }

  Int32 Commands::setStatMod(Object ^object, Int32 StatIndex, Int32 NewStatModValue)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->SetRealStat(StatIndex, short( NewStatModValue )) ));
    }

    return Int32(0);
  }

  Int32 Commands::setRealStat(Object ^object, Int32 StatIndex, Int32 NewRealStatValue)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int( mobile->SetRealStat(StatIndex, short( NewRealStatValue )) ));
    }

    return Int32(0);
  }

  Int32 Commands::getStatAttributeMax(Object ^object, Int32 StatAttributeIndex)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      switch(StatAttributeIndex)
      {
        case 0: return Int32(mobile->GetMaxHP());
        case 1: return Int32(mobile->GetMaxFatigue());
        case 2: return Int32(mobile->GetMaxMana());
      }
    }

    return Int32(0);
  }

  Int32 Commands::setStatAttributeMax(Object ^object, Int32 StatAttributeIndex, Int32 NewMaxValue)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      switch(StatAttributeIndex)
      {
        case 0: return Int32(mobile->SetMaxHP(int(NewMaxValue)));
        case 1: return Int32(mobile->SetMaxFatigue(int(NewMaxValue)));
        case 2: return Int32(mobile->SetMaxMana(int(NewMaxValue)));
      }
    }

    return Int32(0);
  }

  Int32 Commands::setSkillLevel_Unused(Object ^object, Int32 SkillNumber, Int32 NewSkillLevel)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->SetSkillLevel(char(SkillNumber), int(NewSkillLevel)));
    }

    return Int32(0);
  }

  Int32 Commands::addSkillLevel_Unused(Object ^object, Int32 SkillNumber, Int32 SkillGain)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GainSkill(char(SkillNumber), int(SkillGain)));
    }

    return Int32(0);
  }

  Int32 Commands::loseSkillLevel_Unused(Object ^object, Int32 SkillNumber, Int32 SkillLoss)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function (multiply input and output by minus one)
      return Int32(mobile->GainSkill(char(SkillNumber), int(SkillLoss) * -1) * -1);
    }

    return Int32(0);
  }

  Int32 Commands::getSkillLevel(Object ^object, Int32 SkillNumber)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      // NOTE: divide the result by 10
      return Int32(mobile->GetSkillLevelReal(char(SkillNumber), 0) / 10);
    }

    return Int32(0);
  }

  Int32 Commands::getSkillLevelReal(Object ^object, Int32 SkillNumber)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      // NOTE: this call equals the one of getSkillLevelRealStat
      return Int32(mobile->GetSkillLevelReal(char(SkillNumber), 0));
    }

    return Int32(0);
  }

  Int32 Commands::getSkillLevelRealStat(Object ^object, Int32 SkillNumber)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      // NOTE: this call equals the one of getSkillLevelReal
      return Int32(mobile->GetSkillLevelReal(char(SkillNumber), 0));
    }

    return Int32(0);
  }

  Int32 Commands::getSkillLevelNoStat(Object ^object, Int32 SkillNumber)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetModdedBaseSkill(char(SkillNumber)));
    }

    return Int32(0);
  }

  Int32 Commands::getSkillLevelNoStatNoMod(Object ^object, Int32 SkillNumber)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->GetBaseSkill(char(SkillNumber)));
    }

    return Int32(0);
  }

  void Commands::setSkillLevel(Object ^object, Int32 SkillNumber, Int32 NewSkillLevel)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      mobile->SetSkillLevel(char(SkillNumber), int(NewSkillLevel));
    }
  }

  void Commands::addSkillLevel(Object ^object, Int32 SkillNumber, Int32 SkillGain)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      mobile->GainSkill(char(SkillNumber), int(SkillGain));
    }
  }

  void Commands::loseSkillLevel(Object ^object, Int32 SkillNumber, Int32 SkillLoss)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function (multiply input by minus)
      mobile->GainSkill(char(SkillNumber), int(SkillLoss) * -1);
    }
  }

  Int32 Commands::getSkillMod(Object ^object, Int32 SkillNumber)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int(mobile->GetSkillMod(char(SkillNumber))));
    }

    return Int32(0);
  }

  Int32 Commands::setSkillMod(Object ^object, Int32 SkillNumber, Int32 NewSkillModValue)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(int(mobile->SetSkillMod(char(SkillNumber), short(NewSkillModValue))));
    }

    return Int32(0);
  }

  Int32 Commands::modifySkillMod(Object ^object, Int32 SkillNumber, Int32 SkillModModifier)
  {
    if(Mobile ^mobile = dynamic_cast<Mobile ^>(object))
    {
      // Call the mobile's related function
      return Int32(mobile->ModifySkillMod(char(SkillNumber), int(SkillModModifier)));
    }

    return Int32(0);
  }

  String^ Commands::getSkillName(Int32 SkillNumber)
  {
    // Call the internal function
    return Global::Skills[SkillNumber]->Name;
  }

  Int32 Commands::getSkillNumber(String^ SkillName)
  {
    // Call the internal function
    return Global::Skills->FindSkill(SkillName);
  }
}