#include "stdafx.h"

#include "Mobile.h"
#include "Global.h"

namespace SkillSystem {

// For the record, "int" equals "signed int", "short" equals "signed short"
// Sometimes I've used explicit casting for those not familiar with C(++)

// short = 16-bit
// int = 32-bit
// long = 64-bit (for the record only, does not really matter)

int Mobile::Random(int Max)
{
  // NOTE: in OSI's version this functionality is placed inside the ServerObject instead of a MobileObject
  return Randomizer.Next(Max);
}

int Mobile::ModifySkillMod(char SkillNumber, int SkillModModifier)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return 0;

  // NOTE: skill modifiers are internally stored as "signed short"!
  signed int CurrentSkillMod = this->SkillMod[SkillNumber];
  signed int NewSkillMod = CurrentSkillMod + SkillModModifier;

  // Store the new skill modifier
  this->SkillMod[SkillNumber] = (signed short) NewSkillMod;

  // Notify the client that the skill level changed
  Global::Skills->SendSkillChangePacket(this, SkillNumber);

  // Return the actual 'skill mod' modifier
  register int ActualChange = int(NewSkillMod) - int(CurrentSkillMod);
  return ActualChange;
}

int Mobile::GainSkill(char SkillNumber, int SkillGain)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return 0;

  // NOTE: skills are internally stored as "unsigned short"!
  unsigned int CurrentBaseLevel = this->SkillLevel[SkillNumber];
  signed int NewBaseLevel = CurrentBaseLevel + SkillGain;

  // Verify the new range
  if(NewBaseLevel < 0)
    NewBaseLevel = 0;
  if(NewBaseLevel > 65535)
    NewBaseLevel = 65535;
  
  // Store the new skill level
  this->SkillLevel[SkillNumber] = (unsigned short) NewBaseLevel;

  // Notify the client that the skill level changed
  Global::Skills->SendSkillChangePacket(this, SkillNumber);

  // Return the actual skill level gained
  register int ActualChange = int(NewBaseLevel) - int(CurrentBaseLevel);
  return ActualChange;
}

int Mobile::SetSkillLevel(char SkillNumber, int NewSkillLevel)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return 0;

  // NOTE: skills are internally stored as "unsigned short"!
  this->SkillLevel[SkillNumber] = (unsigned short) NewSkillLevel;

  // Return the actual skill level
  return int((unsigned int) this->SkillLevel[SkillNumber]);
}

short Mobile::SetSkillMod(char SkillNumber, short NewSkillModValue)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return 0;

  // NOTE: skill modifiers  are internally stored as "signed short"!
  this->SkillMod[SkillNumber] = (signed short) NewSkillModValue;

  // Return the actual skill modifier
  return short(this->SkillMod[SkillNumber]);
}

short Mobile::GetSkillMod(char SkillNumber)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return 0;

  // Return the skill modifier
  return short(this->SkillMod[SkillNumber]);
}

int Mobile::GetModdedBaseSkill(char SkillNumber)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return 0;

  // Remember, SkillLevel are unsigned shorts, and SkillMod are signed shorts!
  int combined = int( (unsigned int) this->SkillLevel[SkillNumber] ) + int( (signed int) this->SkillMod[SkillNumber] );
  if(combined < 0 || combined > 65000)
    combined = 0;
  return combined;
}

int Mobile::GetBaseSkill(char SkillNumber)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return 0;

  // Remember, SkillLevel are unsigned shorts
  return int( (unsigned int) this->SkillLevel[SkillNumber] );
}

int Mobile::GetCurHP()
{
  // Return the current hit points
  return this->CurHP;
}

int Mobile::GetMaxHP()
{
  // Return the maximum hit points
  return this->CurHP;
}

int Mobile::GetCurFatigue()
{
  // Return the current fatigue
  return this->CurFatigue;
}

int Mobile::GetMaxFatigue()
{
  // Return the maximum fatigue
  return this->CurFatigue;
}

int Mobile::GetCurMana()
{
  // Return the current mana
  return this->CurMana;
}

int Mobile::GetMaxMana()
{
  // Return the maximum Mana
  return this->CurMana;
}

int Mobile::SetCurHP(int NewCurHP, int UnknownAndUnused) // this last parameter exists but is always passed as 0 and is not used by the function itself
{
  // Get the current/original value
  int OriginalCurHP = this->CurHP;

  // Do not allow negative values
  if(NewCurHP < 0)
    NewCurHP = 0;

  // Set
  this->CurHP = NewCurHP;

  // After setting (!?), verify against MaxHP
  if(this->CurHP > this->MaxHP)
    this->CurHP = this->MaxHP;

  // Send a StatAttrChange Packet if nedeed
  if(OriginalCurHP != this->CurHP)
    this->QueueStatAttrPacket(0);

  // Return the new current hit points
  return this->CurHP;
}

int Mobile::SetMaxHP(int NewMaxHP)
{
  // Get the current/original value
  int OriginalMaxHP = this->MaxHP;

  // Set
  this->MaxHP = NewMaxHP;

  // After setting (!?), do not allow negative values
  if(this->MaxHP < 0)
    this->MaxHP = 0;

  // Send a StatAttrChange Packet if nedeed
  if(OriginalMaxHP != this->MaxHP)
    this->QueueStatAttrPacket(0);

  // Return the new maximum hit points
  return this->CurHP;
}

int Mobile::SetCurFatigue(int NewCurFatigue)
{
  // Get the current/original value
  int OriginalCurFatigue = this->CurFatigue;

  // Set
  this->CurFatigue = NewCurFatigue;

  // After setting (!?), verify against MaxFatigue
  if(this->CurFatigue > this->MaxFatigue)
    this->CurFatigue = this->MaxFatigue;

  // After setting (!?), do not allow negative values
  if(NewCurFatigue < 0)
    this->CurFatigue = 0;

  // Send a StatAttrChange Packet if nedeed
  if(OriginalCurFatigue != this->CurFatigue)
    this->QueueStatAttrPacket(2);

  // Return the new current fatigue
  return this->CurFatigue;
}

int Mobile::SetMaxFatigue(int NewMaxFatigue)
{
  // Get the current/original value
  int OriginalMaxFatigue = this->MaxFatigue;

  // Set
  this->MaxFatigue = NewMaxFatigue;

  // After setting (!?), do not allow negative values
  if(this->MaxFatigue < 0)
    this->MaxFatigue = 0;

  // Send a StatAttrChange Packet if nedeed
  if(OriginalMaxFatigue != this->MaxFatigue)
    this->QueueStatAttrPacket(2);

  // Return the new maximum fatigue
  return this->CurFatigue;
}

int Mobile::SetCurMana(int NewCurMana)
{
  // Get the current/original value
  int OriginalCurMana = this->CurMana;

  // Set
  this->CurMana = NewCurMana;

  // After setting (!?), verify against MaxMana
  if(this->CurMana > this->MaxMana)
    this->CurMana = this->MaxMana;

  // After setting (!?), do not allow negative values
  if(NewCurMana < 0)
    this->CurMana = 0;

  // Send a StatAttrChange Packet if nedeed
  if(OriginalCurMana != this->CurMana)
    this->QueueStatAttrPacket(1);

  // Return the new current mana
  return this->CurMana;
}

int Mobile::SetMaxMana(int NewMaxMana)
{
  // Get the current/original value
  int OriginalMaxMana = this->MaxMana;

  // Set
  this->MaxMana = NewMaxMana;

  // After setting (!?), do not allow negative values
  if(this->MaxMana < 0)
    this->MaxMana = 0;

  // Send a StatAttrChange Packet if nedeed
  if(OriginalMaxMana != this->MaxMana)
    this->QueueStatAttrPacket(1);

  // Return the new maximum mana
  return this->CurMana;
}

int Mobile::GetSkillSuccessChance(char SkillNumber, int Diff, int Focus)
{
  // When converting this function, keep in mind:
  // 1) it's a pure integer formula
  // 2) the multiplication result is never auto-casted to a long (64-bit) but stays int (32-bit)
  //    - that's why I added a 'int( )' inside the function to clarify this casting
  return int( (this->GetSkillLevelReal(SkillNumber, 0) - Diff) * 100 ) / Focus + 500;
}

bool Mobile::GetDisplayDebugInfoFlag(char SkillNumber)
{
  // I did not decompile this, as it is not needed for correct functioning of the skill system
  return false;
}

void Mobile::LearnSkillInternal(char SkillNumber, int Chance, int AdvRatePercentage)
{
  // Validate the chance parameter, do nothing when a negative chance is given
  if(Chance <= 0)
    return;
  if(Chance > 1000)
    Chance = 1000;

  // Counselors and GM's can't learn skills
  if(this->IsPlayer())
    if(this->GetFlag4000())
      return;

  // Calculate the advancement rate
  // NOTE: signed multiplication, unsigned division
  int AdvRate = int( (unsigned int) (Global::Skills->GetSkillAdvRate(SkillNumber, this->GetBaseSkill(SkillNumber)) * AdvRatePercentage) / (unsigned int) 100 );

  // Generate debug output
  if(this->GetDisplayDebugInfoFlag(SkillNumber))
  {
    // sprintf(s, "Learn %s At: %d, Chance: %d, AdvRate: %d", Skills.GetSkillName(SkillNumber), this->GetSkillLevel(SkillNumber), Chance, AdvRate);
  }

  // Modify the chance based on the advancement rate
  // NOTE: I added the cast to int to make you understand the compiler does not use a 64-bit intermediate result
  Chance = int(Chance * AdvRate) / 1000;

  // Calculate the skill gain
  int SkillGain = Chance / 1000;

  // Let the randomizer decide if you can gain an extra skill point
  // Note: the above calculation is done again, stupid uh?
  if(Chance / 1000 > Random(1000))
    SkillGain ++;

  // Almost there
  if(SkillGain > 0)
  {
    // Get the skill object
    SkillSystem::Skill ^Skill = Global::Skills->GetSkillObject(SkillNumber);

    // Adjust the skill gain so you can't gain more than 1000 skill points
    // NOTE: this is an unsigned compare whereas the values are signed
    if((unsigned int) (this->GetBaseSkill(SkillNumber) + SkillGain) < 1000)
      SkillGain = 1000 - this->GetBaseSkill(SkillNumber);

    // Go gain the skill
    int SkillPointsGained = this->GainSkill(SkillNumber, SkillGain);
    this->PostSkillGain(SkillNumber, SkillPointsGained);

    // Notify the client that the skill level changed
    Global::Skills->SendSkillChangePacket(this, SkillNumber);
  }
}

int Mobile::TestAndLearnSkill(char SkillNumber, int Diff, int Focus, bool LearnAlways, int AdvRatePercentage, int ChanceModifier)
{
  // Counselors and GM's can't learn skills
  if(this->IsPlayer())
    if(this->GetFlag4000())
      return 0;
 
  // Get the skill's success chance
  int Chance = this->GetSkillSuccessChance(SkillNumber, Diff, Focus);

  // If the chance modifier is one then you are actively using the skill, there is a chance to gain stats!
  if(ChanceModifier == 1)
  {
    this->Tasks[SkillNumber] = Global::GetTimeSecs();
    if(this->IsPlayer())
    {
      // Register...
      Global::Skills->RegisterSkillUsage(SkillNumber);
    }

    // Calculate the AttrMod and call the Mobile's function to gain stats
    // NOTE: signed multiplication, unsigned division
    this->GainStatBySkill(SkillNumber, int( (unsigned int) ( Global::Skills->GetSkillAdvRate(SkillNumber, this->GetBaseSkill(SkillNumber)) * AdvRatePercentage ) / (unsigned int) 10000 ), 1);
  }

  // Calculate the success
  // WARNING! if ChanceModifier == 0 then your server/emulator will crash
  int Success = (Chance / ChanceModifier) - Random(1000);

  // Generate debug output
  if(this->GetDisplayDebugInfoFlag(SkillNumber))
  {
    // sprintf(s, "%s At:(%d/%d), Diff:%d, Focus:%d, Chance:%d, Success:%d", Skills.GetSkillName(SkillNumber), this->GetModdedBaseSkill(SkillNumber), this->GetSkillLevelReal_UseIgnoredBaseSkill(), Diff, Focus, Chance, Success);
  }

  // Can we learn the skill?
  if(LearnAlways || (Success > 0))
  {
    // Go learn the skill
    this->LearnSkillInternal(SkillNumber, 1000 - Chance, AdvRatePercentage);

    // Allow people nearby to learn the skill if your chance to learn the skill was one on one!
    if(ChanceModifier == 1)
    {
      // One can only learn by watching a human
      if(this->IsHuman())
      {
         // Code I didn't decompile fully yet, is related with learning by watching
         // If you really really really need a decompile on this, request it...
         // for each Mobile in range
         //   Mobile->TestAndLearnSkill(SkillNumber, Diff, Focus, false, AdvRatePercentage, 4);
      }
    }
  }

  return Success;
}

void Mobile::TestSkillInternal(char SkillNumber, int GainFactor, bool IsUsingSkill)
{
  // Return now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return;

  // Counselors and GM's can't use skills
  if(this->IsPlayer() && this->GetFlag4000())
    return;

  //
  if(this->Tasks[SkillNumber] == 0)
  {
    this->TaskPercentages[SkillNumber] = 0;
  }
  else
  {
    int TickCountDifferenceWithLastUsage = Global::GetTimeSecs() - this->Tasks[SkillNumber];
    if(TickCountDifferenceWithLastUsage >= 900)
    {
      this->TaskPercentages[SkillNumber] = 0;
    }
    else
    {
      int percentage = int(double(TickCountDifferenceWithLastUsage) / 15.0 * 100.0);
      if(percentage <= int( (unsigned int) this->TaskPercentages[SkillNumber]) )
      {
        this->TaskPercentages[SkillNumber] = 0;
      }
      else
      {
        this->TaskPercentages[SkillNumber] -= (unsigned char) char(percentage);
      }       
    }
  }
  this->TaskPercentages[SkillNumber] ++;
  if(IsUsingSkill)
    this->Tasks[SkillNumber] = Global::GetTimeSecs();
  if(int( (unsigned int) this->TaskPercentages[SkillNumber] ) > 100)
    this->TaskPercentages[SkillNumber] = 100;

  // You can't learn a skill if this test fails! Random skill gain? I love you OSI!
  if( int( (unsigned int) this->TaskPercentages[SkillNumber] ) > Random(100) )
    return;

  // Calculate the chance
  // Resulting value should be in range of 0-1000
  int Chance = int( (1000 - this->GetSkillLevelReal_UseIgnoredBaseSkill(SkillNumber)) * GainFactor ) / 100;

  // Generate debug output
  if(this->GetDisplayDebugInfoFlag(SkillNumber))
  {
    // sprintf(s, "%s At:(%d/%d), AttrMod:%d", Skills.GetSkillName(SkillNumber), this->GetModdedBaseSkill(SkillNumber), this->GetSkillLevelReal_UseIgnoredBaseSkill(), GainFactor / 2);
  }

  // Go learn the skill
  this->LearnSkillInternal(SkillNumber, Chance, 100);

  // You can only gain a stat if you are actively using the skill
  if(IsUsingSkill)
  {
    // Go check for a stat gain
    this->GainStatBySkill(SkillNumber, GainFactor / 2, 1);    

    // Allow others to learn the skill by watching you
    this->HandleWatchingSkill(SkillNumber, GainFactor);
  }
}

void Mobile::HandleWatchingSkill(char SkillNumber, int GainFactor)
{

  // Counselors and GM's can't use skills
  if(this->IsPlayer() && this->GetFlag4000())
    return;

  // One can only learn by watching a human
  if(!this->IsHuman())
    return;

  // Code I didn't decompile fully yet
  // If you really really really need a decompile on this, request it...
  // SkillLevelRealOfHumanBeingWatched = this->GetSkillLevelReal(SkillNumber, 0)
  // for each Mobile
  //   if distance between Mobile and this <= 1
  //     SkillLevelRealOfMobileWatching = Mobile->GetSkillLevelReal_UseIgnoredBaseSkill(SkillNumber)
  //     if SkillLevelRealOfHumanBeingWatched > SkillLevelRealOfMobileWatching
  //       TestGainFactor = int((SkillLevelRealOfHumanBeingWatched - SkillLevelRealOfMobileWatching) * GainFactor) / 1000
  //       Mobile->TestSkillInternal(SkillNumber, TestGainFactor, 0)

}

int Mobile::CalculateSomething(int MaximumStatGain, int StatOfSkill, int StatReqOfSkill, int AttrMod, int RealStat)
{
  //
  if(MaximumStatGain > 0)
    RealStat = 100 - RealStat;
  return int(RealStat * StatOfSkill * AttrMod) / 1000;
}

int Mobile::GoGetRealStat(int StatNumber)
{
  return GetRealStat(StatNumber);
}

short Mobile::GoSetRealStat(int StatIndex, short NewRealStatValue)
{
  return SetRealStat(StatIndex, NewRealStatValue);
}

short Mobile::SetStatAndCheckLimit(int StatNumber, int NewRealStatValue)
{
  short OldStatValue = this->GoGetRealStat(StatNumber);
  short ToReturn = this->GoSetRealStat(StatNumber, NewRealStatValue);
  if( this->CheckStatLimit(StatNumber, NewRealStatValue - OldStatValue) )
    ToReturn = this->GoGetRealStat(StatNumber);
  return ToReturn;
}

void Mobile::GainStatBySkill(char SkillNumber, int AttrMod, int StatGain)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return;

  // Get the skill object
  SkillSystem::Skill ^Skill = Global::Skills->GetSkillObject(SkillNumber);

  int StatIndex;
  int Table1[3], Table2[3];

  // Set Up the internal stat gain tables
  for(StatIndex = 0; StatIndex < 3; StatIndex ++)
  {
     Table1[StatIndex] = this->CalculateSomething(StatGain, Skill->GetStatOfSkill(StatIndex), Skill->GetStatReqOfSkill(StatIndex), AttrMod, (int) this->GetRealStat(StatIndex));
     Table2[StatIndex] = Random(20000);
  }

  // Gain stats according to the chance
  for(StatIndex = 0; StatIndex < 3; StatIndex ++)
  {
    if(Table1[StatIndex] > Table2[StatIndex])
    {
      // Don't allow the real stat to become higher than 100
      if(int( this->GetRealStat(StatIndex) ) + StatGain >= 100)
        StatGain = 0;

      // Modify the real stat
      if(this->IsPlayer())
        this->SetStatAndCheckLimit(StatIndex, int( this->GetRealStat(StatIndex) ) + StatGain);
      else
        ModifyRealStat(StatIndex, short( StatGain ));
    }
  }
}

int Mobile::TestSkill(char SkillNumber, int Unused1, int Unused2)
{
  // Return 0 now if the skill isn't valid/enabled
  if(!Global::Skills->IsSkillEnabled(SkillNumber))
    return 0;

  // If a player uses a skill, register a skill usage
  if(this->IsPlayer())
  {
    // Counselors and GM's can't use skills
    if(this->GetFlag4000())
      return 0;

    // Register...
    Global::Skills->RegisterSkillUsage(SkillNumber);
  }

  // Retreive the related skill object
  // NOTE: This skill object is unused, why?
  SkillSystem::Skill ^Skill = Global::Skills->GetSkillObject(SkillNumber);

  // Get the current skill level and substract a random number of points from it
  int SkillLevelRealMinusRandom1000 = this->GetSkillLevelReal(SkillNumber, 0) - Random(1000);

  // Calculate the "GainFactor"
  int GainFactor = SkillLevelRealMinusRandom1000 < 0 ? 50 : 100;

  // Call the internal Testskill function
  this->TestSkillInternal(SkillNumber, GainFactor, 1);

  // Return that calculated value
  return SkillLevelRealMinusRandom1000;
}

int Mobile::GetSkillLevelReal(char SkillNumber, int Ignored_UseBaseSkillOrModdedBase)
{
  // Get the skill object
  SkillSystem::Skill ^Skill = Global::Skills->GetSkillObject(SkillNumber);

  // Even though the core executes these math functions, it is unneeded for the final result
  // Internal debugging?
  double UnusedSkillLevel = double(Ignored_UseBaseSkillOrModdedBase ? long(this->GetBaseSkill(SkillNumber)) : long(this->GetModdedBaseSkill(SkillNumber)));
  double SkillStatMul10   = long(Skill->SkillStat) * 10.0;
  double UnusedResult     = 1000.0 - SkillStatMul10;

  // Warning! Keep the many castings in mind when converting to another language!
  int Base, Mod;
  Mod = int((unsigned short) this->Str) * Skill->Strength + int((unsigned short) this->Dex) * Skill->Dexterity +  int((unsigned short) this->Int) * Skill->Intelligence;;
  Mod = int( (100 - Skill->SkillStat) * Mod ) / 1000;
  Base = int( (unsigned int)( (1000 - Mod) * this->GetModdedBaseSkill(SkillNumber) ) / 1000 );

  return Base + Mod;
}

int Mobile::GetSkillLevelReal_UseIgnoredBaseSkill(char SkillNumber)
{
  // Pass-Through
  return GetSkillLevelReal(SkillNumber, 1);
}

short Mobile::GetStat(int StatIndex)
{
  // Combine the real stat and the stat modifier to obtain the stat value
  int StatValue = int(this->GetRealStat(StatIndex)) + int(this->GetStatMod(StatIndex));

  // Min/Max check
  if(StatValue < 0)
    return 0;
  if(StatValue > 65000)
    return 0;

  // Done
  return StatValue;
}

short Mobile::GetRealStat(int StatIndex)
{
  // Return the correct real stat determined by the index
  switch(StatIndex)
  {
    case 0: return this->Str;
    case 1: return this->Dex;
    case 2: return this->Int;
  }

  return 0;
}

short Mobile::SetRealStat(int StatIndex, short NewRealStatValue)
{
  // Calculate the difference with the current stat
  int RealStatModifier = int(NewRealStatValue) - int(this->GetRealStat(StatIndex));

  // Set the correct real stat determined by the index
  switch(StatIndex)
  {
    case 0: this->Str = NewRealStatValue;
            this->ModifyMaxHP(RealStatModifier);
            break;
    case 1: this->Dex = NewRealStatValue;
            this->ModifyMaxFatigue(RealStatModifier);
            break;
    case 2: this->Int = NewRealStatValue;
            this->ModifyMaxMana(RealStatModifier);
            break;
  }

  // Notify the client that the stats changed
 // TODO IMPORTANT  GoSendPacket11(this, StatIndex, RealStatModifier);

  // Return the current real stat value
  return this->GetRealStat(StatIndex);
}

short Mobile::ModifyRealStat(int StatIndex, short RealStatModifier)
{
  // Modify the stat
  int OriginalRealStat = int(this->GetRealStat(StatIndex));
  return short( int( this->SetRealStat(StatIndex, OriginalRealStat + int(RealStatModifier)) ) - OriginalRealStat );
}

short Mobile::GetStatMod(int StatIndex)
{
  // Return the correct stat modifier determined by the index
  switch(StatIndex)
  {
    case 0: return this->StrMod;
    case 1: return this->DexMod;
    case 2: return this->IntMod;
  }

  return 0;
}

short Mobile::ModifyStatMod(int StatIndex, short StatModModifier)
{
  // Modify the stat modifier
  int OriginalStatMod = int(this->GetStatMod(StatIndex));
  return short( int( this->SetStatMod(StatIndex, OriginalStatMod + int(StatModModifier)) ) - OriginalStatMod );
}

short Mobile::SetStatMod(int StatIndex, short NewStatModValue)
{
  // Calculate the difference with the current stat modifier
  int StatModModifier = int(NewStatModValue) - int(this->GetStatMod(StatIndex));

  // Set the correct stat modifier determined by the index
  switch(StatIndex)
  {
    case 0: this->StrMod = NewStatModValue;
            this->ModifyMaxHP(StatModModifier);
            break;
    case 1: this->DexMod = NewStatModValue;
            this->ModifyMaxFatigue(StatModModifier);
            break;
    case 2: this->IntMod = NewStatModValue;
            this->ModifyMaxMana(StatModModifier);
            break;
  }

  // Notify the client that the stats changed
 // TODO IMPORTANT  GoSendPacket11(this, StatIndex, StatModModifier);

  // Return the current stat modifier
  return this->GetStatMod(StatIndex);
}

int Mobile::ModifyMaxHP(int ModifierMaxHP)
{
  int CurMaxHP = this->GetMaxHP();
  return this->SetMaxHP(this->GetMaxHP() + ModifierMaxHP) - CurMaxHP;
}

int Mobile::ModifyMaxFatigue(int ModifierMaxFatigue)
{
  int CurMaxFatigue = this->GetMaxFatigue();
  return this->SetMaxFatigue(this->GetMaxFatigue() + ModifierMaxFatigue) - CurMaxFatigue;
}

int Mobile::ModifyMaxMana(int ModifierMaxMana)
{
  int CurMaxMana = this->GetMaxMana();
  return this->SetMaxMana(this->GetMaxMana() + ModifierMaxMana) - CurMaxMana;
}

bool Mobile::IsHuman()
{
  return true;
}

Mobile::Mobile()
{
  _Name = "Flobbitz";

  // NOTE: in the original code this is a fixed-array, this is not supported by Managed C++ for native types
  SkillMod        = new signed short[MAXIMUM_SUPPORTED_SKILLS];
  SkillLevel      = new unsigned short[MAXIMUM_SUPPORTED_SKILLS];
  SkillSlush      = 0;
  Tasks           = new int[MAXIMUM_SUPPORTED_SKILLS];
  TaskPercentages = new unsigned char [MAXIMUM_SUPPORTED_SKILLS];

  // Initialize the arrays
  for(unsigned int i = 0; i < MAXIMUM_SUPPORTED_SKILLS; i ++)
  {
    SkillMod[i]        = 0;
    SkillLevel[i]      = 0;
    Tasks[i]           = Global::GetTimeSecs(); // Original code uses this constant 0x33FA0481 thereby representing a date in 1997
    TaskPercentages[i] = 0;
  }
}

// EIP=0x004566E4
// NOTE: OSI implemented this function only into the PlayerObject
bool Mobile::GetFlag4000()
{
  // We do not have counselors and GM's here ;-)
  return false;
}

void Mobile::QueueStatAttrPacket(int StatIndex)
{
  // TODO
}

// EIP=0x00473450
int Mobile::GetSkillTotal()
{
  // Calculate the total
  int Total = 0;
  for(int SkillNumber = 0; SkillNumber < Global::Skills->Count; SkillNumber ++)
    Total += GetBaseSkill(char(SkillNumber));
  return Total;
}

// EIP=0x004734A4
int Mobile::GetCappedSkillTotal()
{
  // Calculate the total
  int Total = 0;
  for(int SkillNumber = 0; SkillNumber < Global::Skills->Count; SkillNumber ++)
    Total += Global::Skills[SkillNumber]->SkillWeight * GetBaseSkill(char(SkillNumber));
  return Total;
}

void Mobile::HandleHealthGain()
{
  // NOTE: this is not implemented here but...
  if(IsPlayer())
  {
    // If the player is at full health, then the "canReportIdList" and "canReportNameList" variables are removed
    // EIP=0x004564E6
  }
  else
  {
    // For all other mobiles this function does nothing
    // EIP=0x004723E9
  }
}

int Mobile::GetRandomStatNumberToModify(int StatNumberProtectedFromChange)
{
  int StatIf0, StatIf1;

  StatIf1 = 0;
  StatIf0 = 2;
  switch(StatNumberProtectedFromChange)
  {
  case 0:
    StatIf1 = 2;
    StatIf0 = 1;
    break;
  case 1:
    StatIf1 = 2;
    StatIf0 = 0;
    break;
  case 2:
    StatIf1 = 0;
    StatIf0 = 1;
    break;
  }

  // Return
  int StatToModify;
  if(this->Random(2)) // 0 or 1
    StatToModify = StatIf1;
  else
    StatToModify = StatIf0;
  return StatToModify;
}

int Mobile::ModifyAnyOtherStat(int StatNumberProtectedFromChange, int StatChange)
{
  int StatNumberToChange = this->GetRandomStatNumberToModify(StatNumberProtectedFromChange);
  int NewStatValue = this->GoGetRealStat(StatNumberToChange);
  return this->GoSetRealStat(StatNumberToChange, (short)(NewStatValue + StatChange));
}

int Mobile::CheckStatLimit(int StatNumber, int StatChange)
{
  int ChanceToDropStat;

  // Calculate the StatTotal
  int StatTotal = 0;
  for(int StatCounter = 0; StatCounter < 3; StatCounter ++)
    StatTotal += (int)(unsigned int)this->GetRealStat(StatCounter);

  //
  if(StatTotal > 225)
  {
    // Stat Total reached! Drop points asap!
    int ExcessStatPoints = StatTotal - 225;
    ModifyAnyOtherStat(StatNumber, -ExcessStatPoints);
    return 1;
  }
  else
  {
    if(StatChange > 0)
    {
       // Stat Total not yet reached and we gained atleast one stat! There is a (small) chance to drop another stat... :(
      double dStatTotal = (double) StatTotal;
      if(dStatTotal > Global::StatPointsHigh_ForCompare)
        ChanceToDropStat = int((dStatTotal - Global::StatPointsHigh_ForSubstract) * (dStatTotal - Global::StatPointsHigh_ForSubstract) * Global::StatFormula_HighToMax + Global::StatCurveHigh);
      else
        ChanceToDropStat = int((dStatTotal - Global::StatPointsLow) * (dStatTotal - Global::StatPointsLow) * Global::StatFormula_LowToHigh + Global::StatCurveLow);
      for(int i = 0; i < StatChange; i ++)
      {
        if(Random(1000) < ChanceToDropStat)
          ModifyAnyOtherStat(StatNumber, -1);
      }
    }
    return 0;
  }
}

void Mobile::ModifySomeOtherSkill(int SkillNumberProtectedFromChange, int SkillPointsToDrop)
{
  // Negate (the input is actually SkillPointsToGain but is negated here)
  // Why? Ask Raph Koster and co!
  SkillPointsToDrop = -SkillPointsToDrop;

  //
  if(this->SkillSlush - SkillPointsToDrop >= 0)
  {
    this->SkillSlush -= SkillPointsToDrop;
  }
  else
  {
    List<int> SkillsWeCanDropFrom = gcnew List<int>();
    
    // Put all skills we can drop in an array list
    int TotalSkillCount = Global::Skills->GetTotalSkillCount();
    for(int SkillCounter = 0; SkillCounter < TotalSkillCount; SkillCounter ++)
    {
      if(SkillCounter != SkillNumberProtectedFromChange)
        if(this->GetBaseSkill(SkillCounter) >= SkillPointsToDrop)
          SkillsWeCanDropFrom.Add(SkillCounter);
    }

    //
    int NumberOfSkillsWeCanDropFrom = SkillsWeCanDropFrom.Count;
    if(NumberOfSkillsWeCanDropFrom != 0)
    {
      //
      int TimeDiffMinutesTotalOfSkillsWeCanDrop = 0;
      for(int i = 0; i < NumberOfSkillsWeCanDropFrom; i ++)
        TimeDiffMinutesTotalOfSkillsWeCanDrop += (Global::GetTimeSecs() - this->Tasks[SkillsWeCanDropFrom[i]]) / 60;

      //
      int j, MinutesLeftUntilDropSkill = 0;
      if(TimeDiffMinutesTotalOfSkillsWeCanDrop > 0)
        MinutesLeftUntilDropSkill = Global::GetRandomizedTaskTotalInMinutes(0, TimeDiffMinutesTotalOfSkillsWeCanDrop);
      for(j = 0; ; j ++)
      {
        if(j < NumberOfSkillsWeCanDropFrom - 1)
        {
          MinutesLeftUntilDropSkill -= (Global::GetTimeSecs() - this->Tasks[SkillsWeCanDropFrom[j]]) / 60;
          if(MinutesLeftUntilDropSkill > 0)
            continue;
        }
        break;
      }

      //
      int SkillNumberToDrop = SkillsWeCanDropFrom[j];
      this->GetBaseSkill(SkillNumberToDrop);
      this->GainSkill(SkillNumberToDrop, -SkillPointsToDrop);
      int SkillSlushModifier = Global::Skills->GetSkillObject(SkillNumberToDrop)->SkillWeight - SkillPointsToDrop;
      if(SkillSlushModifier > 0)
        this->SkillSlush += SkillSlushModifier;
      Global::Skills->SendSkillChangePacket(this, SkillNumberToDrop);
    }
    else
    {
      // LOL, no skills to drop from, so undo the gain we just had... Haha, you are screwed dude!
      if(this->GetBaseSkill(SkillNumberProtectedFromChange) < SkillPointsToDrop)
        this->SetSkillLevel(SkillNumberProtectedFromChange, 0);
      else
        this->GainSkill(SkillNumberProtectedFromChange, -SkillPointsToDrop);
      Global::Skills->SendSkillChangePacket(this, SkillNumberProtectedFromChange);
    }
  }
}

int Mobile::PostSkillGain(int SkillNumber, int SkillPointsGained)
{
  if(this->IsPlayer())
  {
    // Not decompiled 100% here but : editors, counselors, GM's well never loose skill on gains
    if(this->GetFlag4000())
      return 0;
  }

  int CappedSkillTotal = this->GetCappedSkillTotal();
  if(this->IsPlayer() && CappedSkillTotal > 7000 )
  {
    for(int i = CappedSkillTotal - 7000; i != 0; i --)
      this->ModifySomeOtherSkill(SkillNumber, -1);
    return 1;
  }
  else
  {
    if(SkillPointsGained > 0)
    {
      int ChanceToDropSkill = Global::ApplySkillCurve(CappedSkillTotal);
      for(int i = 0; i < SkillPointsGained; i ++)
      {
        if(Random(1000) < ChanceToDropSkill)
          this->ModifySomeOtherSkill(SkillNumber, -1);
      }
    }
    return 0;
  }
}

}