// Mobile.h

#pragma once

#include "DynamicItem.h"
#include "Skills.h"

using namespace System;

namespace SkillSystem
{
  #define MOBILEPROPERY(type, name) \
    property type name \
    { \
      type get()       { return _##name           ; } \
      void set(type __##name) { _##name = __##name; } \
    } \

public ref class Mobile : public DynamicItem
{
private:
  static Random Randomizer;
  static int Random(int Max);

protected: // Property Variables
   String ^_Name;

  short _Str, _Dex, _Int;
  short _StrMod, _DexMod, _IntMod;

protected: // Variables
  signed short *SkillMod;
  unsigned short *SkillLevel;
  int SkillSlush;

  int *Tasks;
  unsigned char *TaskPercentages;
  
  int MaxHP, CurHP;
  int MaxMana, CurMana;
  int MaxFatigue, CurFatigue;

public:
  MOBILEPROPERY(String^, Name);

  MOBILEPROPERY(short, Str);
  MOBILEPROPERY(short, Dex);
  MOBILEPROPERY(short, Int);
  MOBILEPROPERY(short, StrMod);
  MOBILEPROPERY(short, DexMod);
  MOBILEPROPERY(short, IntMod);

public: // Attributes  
  int GetMaxHP();
  int GetMaxFatigue();
  int GetMaxMana();

  int SetMaxHP(int NewMaxHP);
  int SetMaxMana(int NewMaxMana);
  int SetMaxFatigue(int NewMaxFatigue);

  int GetCurHP();
  int GetCurFatigue();
  int GetCurMana();
  
  int SetCurHP(int NewCurHP, int UnknownAndUnused); // this last parameter exists but is always passed as 0 and is not used by the function itself
  int SetCurFatigue(int NewCurFatigue);
  int SetCurMana(int NewCurMana);

  void HandleHealthGain();

private: // Attributes
  int ModifyMaxHP(int ModifierMaxHP);
  int ModifyMaxFatigue(int ModifierMaxFatigue);
  int ModifyMaxMana(int ModifierMaxMana);
  
public: // Stats
  short GetStat(int StatIndex);
  short GetRealStat(int StatIndex);
  short SetRealStat(int StatIndex, short NewRealStatValue);
  short ModifyRealStat(int StatIndex, short RealStatModifier);
  short GetStatMod(int StatIndex);
  short ModifyStatMod(int StatIndex, short StatModModifier);
  short SetStatMod(int StatIndex, short NewStatModValue);

private: // Stats
  int GoGetRealStat(int StatIndex);
  short GoSetRealStat(int StatIndex, short NewRealStatValue);
  int GetRandomStatNumberToModify(int StatNumberProtectedFromChange);
  int ModifyAnyOtherStat(int StatNumberProtectedFromChange, int StatChange);
  int CheckStatLimit(int StatNumber, int StatChange);
  short SetStatAndCheckLimit(int StatNumber, int NewRealStatValue);
  void GainStatBySkill(char SkillNumber, int AttrMod, int StatGain);

public: // Skills
  int GetSkillTotal();
  int GetCappedSkillTotal();

  int GainSkill(char SkillNumber, int SkillGain);;
  int SetSkillLevel(char SkillNumber, int NewSkillLevel);
  short SetSkillMod(char SkillNumber, short NewSkillModValue);
  short GetSkillMod(char SkillNumber);
  int ModifySkillMod(char SkillNumber, int SkillModModifier);
  int GetModdedBaseSkill(char SkillNumber);
  int GetBaseSkill(char SkillNumber);
  int GetSkillLevelReal(char SkillNumber, int Ignored_UseBaseSkillOrModdedBase);
  int GetSkillLevelReal_UseIgnoredBaseSkill(char SkillNumber);

  int GetSkillSuccessChance(char SkillNumber, int Diff, int Focus);
  int TestSkill(char SkillNumber, int Unused1, int Unused2);
  int TestAndLearnSkill(char SkillNumber, int Diff, int Focus, bool LearnAlways, int AdvRatePercentage, int ChanceModifier);
  void HandleWatchingSkill(char SkillNumber, int GainFactor);

private: // Skills
  void LearnSkillInternal(char SkillNumber, int Chance, int AdvRatePercentage);
  void TestSkillInternal(char SkillNumber, int GainFactor, bool IsUsingSkill);
  int CalculateSomething(int MaximumStatGain, int StatOfSkill, int StatReqOfSkill, int AttrMod, int RealStat);
  void ModifySomeOtherSkill(int SkillNumberProtectedFromChange, int SkillChange);
  int PostSkillGain(int SkillNumber, int SkillPointsGained);

private: // Other
  bool GetFlag4000();
  bool GetDisplayDebugInfoFlag(char SkillNumber);
  void QueueStatAttrPacket(int StatIndex);

public:
  virtual bool IsHuman() override;

public:
  Mobile();
};

}