// Skill.h

#pragma once

#include <math.h>

using namespace System;

namespace SkillSystem
{
  #define SKILLPROPERTY(type, name) \
    property type name \
    { \
      type get()       { return _##name           ; } \
      void set(type __##name) { _##name = __##name; } \
    } \

  public ref class Double3
  {
  private:
    double _A, _B, _C;

  public:
    SKILLPROPERTY( double, A );
    SKILLPROPERTY( double, B );
    SKILLPROPERTY( double, C );

    void Init(double skillmax, double lower, double middle, double higher)
    {
      double l_min_h = lower - higher;
      double m_min_l = middle - lower;
      double h_min_l = higher - lower;
      double h_min_m = higher - middle;

      _A = h_min_l;

      double f = sqrt((l_min_h * l_min_h) - (m_min_l * 4.0 * h_min_m));
      double m_min_l_times_2 = m_min_l * 2.0;
      double a_plus_f_div_x  = (_A + f) / m_min_l_times_2;
      double a_minus_f_div_x = (_A - f) / m_min_l_times_2;
      double realA = fabs(a_minus_f_div_x - 1.0) < fabs(a_plus_f_div_x - 1.0) ? a_plus_f_div_x : a_minus_f_div_x;

      _A = realA;

      _A = pow(_A, 2.0 / skillmax);

      _B = h_min_l;
      _B = _B / (pow(_A, skillmax) - 1.0);

      _C = lower - _B;
    }

    double ApplyAdvRate( double BaseSkill )
    {
      return Math::Pow( _A, BaseSkill ) * _B + _C;
    }
  };

  public ref class Skill
  {
  private:
    String ^_Name,  ^_Script;
    int _Strength,  _Dexterity,   _Intelligence;
    int _StrReq,    _DexReq,      _IntReq;
    int _AdvRateA,  _AdvRateB,    _AdvRateC, _StatAdvRate;
    int _SkillStat, _SkillWeight, _Version;
    bool _CanUse;

  public:
    Double3 ^Double3;

  public:
    SKILLPROPERTY( String^, Name         );
    SKILLPROPERTY( int,     Strength     );
    SKILLPROPERTY( int,     Dexterity    );
    SKILLPROPERTY( int,     Intelligence );
    SKILLPROPERTY( int,     StrReq       );
    SKILLPROPERTY( int,     DexReq       );
    SKILLPROPERTY( int,     IntReq       );
    SKILLPROPERTY( int,     AdvRateA     );
    SKILLPROPERTY( int,     AdvRateB     );
    SKILLPROPERTY( int,     AdvRateC     );
    SKILLPROPERTY( int,     StatAdvRate  );
    SKILLPROPERTY( int,     SkillStat    );
    SKILLPROPERTY( bool,    CanUse       );
    SKILLPROPERTY( int,     SkillWeight  );
    SKILLPROPERTY( String^, Script       );
    SKILLPROPERTY( int,     Version      );

    Skill();

    bool IsSkillEnabled();
    int GetStatOfSkill(int StatIndex);
    int GetStatReqOfSkill(int StatIndex);
	};
}
