#include "stdafx.h"

#include "Skills.h"

namespace SkillSystem {

Skills::Skills()
{
  PerSkillSomeValue = gcnew array<unsigned int>(MAXIMUM_SUPPORTED_SKILLS);
  AllSkillSomeTotal = 0;
  PerSkillUsageCounter = gcnew array<unsigned int>(MAXIMUM_SUPPORTED_SKILLS);
  AllSkillUsageCounter = 0;

  // NOTE: MAXIMUM_SUPPORTED_SKILLS is not enforced
  SkillList.Capacity = MAXIMUM_SUPPORTED_SKILLS;
}

int Skills::Count::get()
{
  return SkillList.Count;
}

void Skills::Clear()
{
  SkillList.Clear();
}

Skill^ Skills::operator[](int i)
{
  if(i < 0 || i >= SkillList.Count)
    return nullptr;
  return SkillList[i];
}

int Skills::AddSkill(Skill^ Skill)
{
  SkillList.Add(Skill);
  return SkillList.Count - 1;
}

int Skills::FindSkill(String^ Name)
{
  for(int i = 0; i < SkillList.Count; i ++)
    if(SkillList[i]->Name == Name)
      return i;
  return -1;
}

int Skills::GetSkillAdvRate(char SkillNumber, int BaseSkill)
{
  unsigned int CombinedCountersAverage = GetCombinedCounters() / GetTotalSkillCount();
  unsigned int CombinedValue = GetCombinedValue(SkillNumber);
  int Rate = (int)((unsigned int)(int(CombinedCountersAverage) * 1000) / (CombinedValue + 1));
  if(Rate < 500)
    Rate = 500;
  else if(Rate > 5000)
    Rate = 5000;

  return (int) ((Rate * GetSkillObject(SkillNumber)->Double3->ApplyAdvRate(BaseSkill)) / 1000.0);
}

void Skills::RegisterSkillUsage(char SkillNumber)
{
  if(!IsSkillEnabled(SkillNumber))
    return;
  
  if(AllSkillSomeTotal + AllSkillUsageCounter == -1) // 0xFFFFFFFF or 4294967295
  {
    AllSkillSomeTotal = 0;
    AllSkillUsageCounter = 0;

    for(int i = 0; i < MAXIMUM_SUPPORTED_SKILLS; i ++)
    {
      PerSkillUsageCounter[i] = PerSkillUsageCounter[i] >> 16;
      PerSkillSomeValue[i]    = PerSkillSomeValue[i]    >> 16;

      AllSkillUsageCounter += PerSkillUsageCounter[i];
      AllSkillSomeTotal    += PerSkillSomeValue[i];
    }
  }

  PerSkillUsageCounter[SkillNumber] ++;
  AllSkillUsageCounter ++;
}

void Skills::SendSkillChangePacket(Mobile ^Mobile, char SkillNumber)
{
  // TODO
}

}